<?php

namespace Drupal\domain_route_redirect;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\domain\DomainInterface;

/**
 * Class DomainRouteRedirectMatcherService.
 */
class DomainRouteRedirectMatcherService {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Path\PathMatcherInterface definition.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Constructs a new DomainRouteRedirectMatcherService object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, PathMatcherInterface $path_matcher) {
    $this->entityTypeManager = $entity_type_manager;
    $this->pathMatcher = $path_matcher;
  }

  /**
   * Match the path and route against all domain_route_redirect entry. Return
   * the respective domain if a matching entry is found.
   */
  public function find_match(DomainInterface $domain, string $path, string $route) {
    if(empty($path) && empty($route)) {
      return NULL;
    }

    // Load all domain_route_redirect objects.
    $domain_route_redirects = $this->entityTypeManager->getStorage('domain_route_redirect')->loadByDomain($domain);

    foreach ($domain_route_redirects as $domain_route_redirect) {
      $allowed_paths = $domain_route_redirect->getRedirectPaths();
      $allowed_routes = $domain_route_redirect->getRedirectRoutes();

      if ( (!empty($path)  && $this->checkPathMatch( $path,  $allowed_paths ))
      ||   (!empty($route) && $this->checkRouteMatch($route, $allowed_routes)) ) {
        return $domain_route_redirect;
      }
    }

    return NULL;
  }

  /**
   * Checks whether a path matches the paths to be redirected.
   */
  private function checkPathMatch(string $path, $allowed_paths) {
    return $this->pathMatcher->matchPath($path, $allowed_paths);
  }

  /**
   * Checks whether a route matches the routes to be redirected.
   */
  private function checkRouteMatch(string $route, $allowed_routes) {
    return in_array($route, $allowed_routes);
  }

}
